/*
 * Copyright 2019 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.drools.workbench.screens.scenariosimulation.client.rightpanel;

import java.util.Objects;

import javax.annotation.PostConstruct;

import com.google.gwt.user.client.ui.Widget;
import org.uberfire.backend.vfs.ObservablePath;
import org.uberfire.client.annotations.DefaultPosition;
import org.uberfire.client.annotations.WorkbenchPartTitle;
import org.uberfire.client.annotations.WorkbenchPartView;
import org.uberfire.workbench.model.CompassPosition;
import org.uberfire.workbench.model.Position;

public abstract class AbstractSubDockPresenter<T extends SubDockView> implements SubDockView.Presenter {

    public static final int DEFAULT_PREFERRED_WIDHT = 300;


    protected String title;
    protected T view;
    protected ObservablePath currentPath;

    public AbstractSubDockPresenter() {
    }

    public AbstractSubDockPresenter(T view) {
        this.view = view;
    }

    @PostConstruct
    public void setup() {
        view.init(this);
    }

    @DefaultPosition
    public Position getDefaultPosition() {
        return CompassPosition.EAST;
    }

    @WorkbenchPartTitle
    public String getTitle() {
        return title;
    }

    @WorkbenchPartView
    public Widget asWidget() {
        return view.asWidget();
    }

    @Override
    public void setCurrentPath(ObservablePath path) {
        currentPath = path;
    }

    @Override
    public boolean isCurrentlyShow(ObservablePath path) {
        return Objects.equals(currentPath, path);
    }
}
